//---------------------------------------------------------------------------
#include <stdio.h>
#include "windows.h"
#pragma hdrstop

#include "serW32.h"
#include "FTD2XX.h"
#include "FTChipId.h"

//---------------------------------------------------------------------------

#define OPTO_BUFFER_SIZE	(16384 *2 + 64)
#define PACKET_SIZE			 4096

typedef struct {
    FT_HANDLE       ftHandle;
    DWORD           chipID;
    char            SerialNumber[16];
    char            Description[64];
    unsigned int    ReadTimeout;
	unsigned int    WriteTimeout;
	int 			commNr;
	int				HandleValid;
	int				opto;
	unsigned char 	opto_read_buffer[OPTO_BUFFER_SIZE];
	unsigned char 	opto_write_buffer[OPTO_BUFFER_SIZE];
} commDataType;



//---------------------------------------------------------------------------

static int comm_reopen(commDataType  * ptr)
{
   FT_STATUS       ftStatus;
    
   ptr->opto	 = 0;
   ptr->ftHandle = NULL;
  ftStatus		 =  FT_OpenEx((void*)"Step2Cnc Motion Controller",FT_OPEN_BY_DESCRIPTION,&ptr->ftHandle);

   if (ftStatus == FT_OK)
   {
	   ftStatus = FTID_GetChipIDFromHandle(ptr->ftHandle, &ptr->chipID);
	   if (ftStatus != FT_OK)
	   {
		   ptr->chipID = 0;
	   }

	   ftStatus = FT_SetUSBParameters (ptr->ftHandle,PACKET_SIZE,0);

	   ptr->ReadTimeout  = 100;	
	   ptr->WriteTimeout = 5;

	   FT_SetTimeouts(ptr->ftHandle,ptr->ReadTimeout,ptr->WriteTimeout);

	   return 0;
   }
   else
   {
       return -1;
   }
}


//---------------------------------------------------------------------------

int comm_Open( void ** handle,int commNr )
{
   commDataType  * ptr = NULL;

   int 				result;

   // allocate data for COM processing

   ptr = new  commDataType;

   if(ptr == NULL)
   {
		return -1;
   }
   ptr->commNr 	 = commNr;

   if (comm_reopen(ptr) != 0)
   {
	  delete ptr;
	  ptr    = NULL;
	  result = -1;
   }
   else
   {
	   *handle		    = (void *)ptr;
	   ptr->HandleValid = 1;
	   result 			= 0;
   }

   return result;
}


//---------------------------------------------------------------------------
int comm_Close(void ** handle)
{
   commDataType  * ptr = (commDataType  *)*handle;

   if(*handle == NULL)
   {
	 return FALSE;
   }

   if(ptr->HandleValid != 0)
   {
	   ptr->HandleValid = 0;
	   FT_Close(ptr->ftHandle);
   }

   return ( TRUE ) ;

}

//---------------------------------------------------------------------------
int comm_Read(void * handle,char * lpBlock, int nMaxLength, int timeout )
{
   DWORD       dwLength;
   char      * buffer;
   int         buffer_len;
   FT_STATUS   ftStatus; 	


   commDataType  * ptr = (commDataType  *)handle;


   if(ptr == NULL)
   {
	  return -1;
   }

   if(ptr->HandleValid == 0)
   {
		 Sleep(500);
		 if(comm_reopen(ptr) == 0)
		 {
           ptr->HandleValid = 1;
         }
   }


   if(ptr->HandleValid != 0)
   {

	   if(timeout != (int)ptr->ReadTimeout)
	   {
		  ptr->ReadTimeout = timeout;
		  FT_SetTimeouts(ptr->ftHandle,ptr->ReadTimeout,ptr->ReadTimeout);
	   }

	   if(ptr->opto)
	   {
		   buffer			= (char*)ptr->opto_read_buffer;
		   buffer_len		= sizeof(ptr->opto_read_buffer);
	   }
	   else
	   {
		   buffer			= lpBlock;
		   buffer_len		= nMaxLength;
	   }

	   ftStatus = FT_Read(ptr->ftHandle,buffer,buffer_len,&dwLength);
	   if(ftStatus == FT_OK)
	   {

		   if(ptr->opto == 0)
		   {
				return dwLength;
		   }
		   else
		   {
				return opto2nrz(buffer,dwLength,lpBlock,nMaxLength);
		   }
	   }
	   else
	   {
			return -1;
	   }
   }
   else
   {
       return -1;
   }
}

//---------------------------------------------------------------------------

int comm_Write(void * handle,const char * lpBlock , unsigned long dwBytesToWrite)
{
  commDataType  * ptr = (commDataType  *)handle;
  unsigned long   bytesWritten;
  FT_STATUS       ftStatus;
  DWORD			  dwLength;

  int 			  result = 0;
  int			  size   = 0;
  int			  ii     = 0;

  if(ptr == NULL)
  {
	 return -1;
  }

  if(ptr->HandleValid != 0)
  {
	  if(FT_Write(ptr->ftHandle,(void *)lpBlock,dwBytesToWrite,&bytesWritten)!= FT_OK)
	  {
		 result = -1;
	  }
  }
  else
  {
	 result = -1;
  }

  return result;
}

//---------------------------------------------------------------------------

void comm_Purge(void * handle)
{
  commDataType  * ptr = (commDataType  *)handle;

  if(ptr == NULL)
  {
     return;
  }

  if(ptr->HandleValid != 0)
  {
	  FT_Purge(ptr->ftHandle,FT_PURGE_RX |FT_PURGE_TX);
  }
}

//---------------------------------------------------------------------------

int   comm_GetID(void * handle,unsigned long * HwId)
{

   commDataType  * ptr = (commDataType  *)handle;

   if(ptr == NULL)
   {
     return -1;
   }

   if(ptr->chipID != 0)
   {
      *HwId = ptr->chipID;
      return 0;
   }
   else
   {
      return -1;
   }

}


